!function (a, b) {
    if ("object" == typeof module && "object" == typeof module.exports) {
        var c = require("jquery");
        module.exports = b(c)
    } else "function" == typeof define && define.amd ? define(["jQuery"],
        function (a) {
            return b(a)
        }) : a.highlighter = a.highlighter || b(a.jQuery)
}(this,
    function ($) {
        // 核心变量定义
        var that = this,
            cidKey = "_mh_cid",
            highlightClass = "chrome-extension-mutihighlight",
            stylePrefix = "chrome-extension-mutihighlight-style-",
            utils = {},
            timeouts = {},
            nodeData = new WeakMap(),
            BATCH_THRESHOLD = 1000,    // 批处理阈值
            BATCH_SIZE = 50,          // 批处理大小
            LOAD_THRESHOLD = 200,     // 加载阈值
            BATCH_INTERVAL = 8;       // 批处理间隔时间

        // 工具函数
        utils.isContentEditable = e => e && (e.parentNode?.isContentEditable || e.isContentEditable);
        utils.isUnderContentEditableOrStyleScript = (node, root) => {
            while (node && node !== root) {
                if (node.tagName && (node.isContentEditable || 
                    ["style", "script"].includes(node.tagName.toLowerCase()))) return true;
                node = node.parentNode;
            }
            return false;
        };
        utils.isScriptOrStyle = e => e.tagName && (
            ["style", "script"].includes(e.tagName.toLowerCase()) || 
            ["style", "script"].includes(e.parentNode?.tagName?.toLowerCase())
        );
        utils.isChildNode = (node, parent) => Array.from(parent.childNodes).includes(node);
        utils.isWrapNode = e => e === document || e === document.body;
        utils.cid = (() => {
            let count = 0;
            return () => ++count;
        })();

        // 优化的文本节点合并函数
        utils.mergeTextNodes = (highlight) => {
            const prev = highlight.previousSibling;
            const next = highlight.nextSibling;
            const text = highlight.textContent;
            
            if (prev && next && prev.nodeType === Node.TEXT_NODE && next.nodeType === Node.TEXT_NODE) {
                prev.nodeValue = prev.nodeValue + text + next.nodeValue;
                next.remove();
            } else if (prev && prev.nodeType === Node.TEXT_NODE) {
                prev.nodeValue = prev.nodeValue + text;
            } else if (next && next.nodeType === Node.TEXT_NODE) {
                next.nodeValue = text + next.nodeValue;
            } else if (highlight.parentNode) {
                highlight.parentNode.insertBefore(document.createTextNode(text), highlight);
            }
            highlight.remove();
        };

        // 修改节点过滤函数
        utils.shouldProcessNode = (node) => {
            if (!node || !node.parentElement) return false;
            
            const parent = node.parentElement;
            if (parent.classList?.contains(highlightClass)) return false;
            if (parent.tagName && ['SCRIPT', 'STYLE', 'TEXTAREA'].includes(parent.tagName)) return false;
            
            const text = node.nodeValue;
            return text && text.trim().length > 0;
        };

        function clearTimeouts() {
            for (var key in timeouts) clearTimeout(timeouts[key]);
        }

        // 核心高亮实现
        var highlightNode, removeHighlight, updateHighlights;

        highlightNode = function(element, keywords, force) {
            if (!Array.isArray(keywords) || !keywords.length) return;

            try {
                const processedKeywords = keywords
                    .filter(item => item?.words && typeof item.words === 'string' && item.words.trim())
                    .sort((a, b) => b.words.length - a.words.length);

                if (!processedKeywords.length) return;

                // 使用 NodeIterator 替代 TreeWalker
                const iterator = document.createNodeIterator(
                    element,
                    NodeFilter.SHOW_TEXT,
                    {
                        acceptNode: function(node) {
                            if (!utils.shouldProcessNode(node)) {
                                return NodeFilter.FILTER_REJECT;
                            }
                            return NodeFilter.FILTER_ACCEPT;
                        }
                    }
                );

                const textNodes = [];
                let currentNode;
                while (currentNode = iterator.nextNode()) {
                    textNodes.push(currentNode);
                }

                // 修改批处理大小设置
                const totalNodes = textNodes.length;
                const batchSize = totalNodes > BATCH_THRESHOLD ? 50 : totalNodes;
                let index = 0;

                function processBatch() {
                    const fragment = document.createDocumentFragment();
                    const end = Math.min(index + batchSize, textNodes.length);
                    
                    for (let i = index; i < end; i++) {
                        const node = textNodes[i];
                        if (!node || !node.parentNode) continue;

                        const text = node.nodeValue;
                        if (!text || !text.trim()) continue;

                        let matches = [];
                        let usedRanges = new Set();
                        
                        processedKeywords.forEach(keyword => {
                            if (!keyword?.words) return;
                            
                            const escapedWord = keyword.words.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                            const isEnglishWord = /^[A-z_-]+$/.test(keyword.words);
                            const pattern = isEnglishWord ? 
                                new RegExp(`(?:^|(?<=[^A-Za-z]))${escapedWord}(?=[^A-Za-z]|$)`, 'gi') :
                                new RegExp(escapedWord, 'g');
                            
                            let match;
                            while ((match = pattern.exec(text)) !== null) {
                                const matchText = match[0];
                                const start = match.index;
                                const end = start + matchText.length;
                                
                                let shouldHighlight = true;
                                for (let range of usedRanges) {
                                    const [usedStart, usedEnd] = range.split(',').map(Number);
                                    if (!(end <= usedStart || start >= usedEnd)) {
                                        shouldHighlight = false;
                                        break;
                                    }
                                }

                                if (shouldHighlight) {
                                    matches.push({
                                        start: start,
                                        end: end,
                                        text: matchText,
                                        keyword: keyword
                                    });
                                    usedRanges.add(`${start},${end}`);
                                }
                            }
                        });

                        if (matches.length) {
                            matches.sort((a, b) => a.start - b.start);
                            let lastIndex = 0;
                            
                            matches.forEach(match => {
                                if (match.start > lastIndex) {
                                    fragment.appendChild(
                                        document.createTextNode(text.slice(lastIndex, match.start))
                                    );
                                }
                                
                                const highlight = document.createElement('mark');
                                highlight.className = `${highlightClass} ${stylePrefix}${match.keyword.colour || 1}`;
                                highlight.textContent = match.text;
                                highlight.setAttribute('data-highlight-text', match.text);
                                highlight.setAttribute('data-category-index', match.keyword.categoryIndex);
                                
                                fragment.appendChild(highlight);
                                lastIndex = match.end;
                            });

                            if (lastIndex < text.length) {
                                fragment.appendChild(
                                    document.createTextNode(text.slice(lastIndex))
                                );
                            }

                            if (node.parentNode && document.contains(node)) {
                                node.parentNode.replaceChild(fragment, node);
                            }
                        }
                    }
                    
                    index = end;
                    if (index < textNodes.length) {
                        requestAnimationFrame(processBatch);
                    }
                }

                processBatch();

            } catch (error) {
                console.error('Error in highlightNode:', error);
            }
        };

        // 添加节点类型记录
        const nodeTypes = new WeakMap();

        // 在高亮时记录原始节点类型
        function highlightNode(node, keywords) {
            // 创建文档片段
            const fragment = document.createDocumentFragment();
            
            // 收集所有需要高亮的节点
            const nodesToHighlight = [];
            
            // 在文档片段中进行高亮处理
            nodesToHighlight.forEach(node => {
                const highlight = document.createElement('mark');
                highlight.className = `${highlightClass} ${stylePrefix}${node.keyword.colour}`;
                highlight.textContent = node.text;
                
                // 保留节点类型记录
                nodeTypes.set(highlight, {
                    type: node.nodeType,
                    content: node.textContent,
                    tagName: node.tagName?.toLowerCase()
                });
                
                fragment.appendChild(highlight);
            });
            
            // 一次性插入DOM
            node.parentNode.replaceChild(fragment, node);
        }

        // 修改 removeHighlight 函数
        removeHighlight = function(element) {
            if (!element) return;
            try {
                // 清理 observer
                const observer = nodeData.get(element);
                if (observer) {
                    observer.disconnect();
                    nodeData.delete(element);
                }
                
                // 直接使用合并逻辑
                utils.mergeTextNodes(element);
            } catch (error) {
                console.error('Error in removeHighlight:', error);
            }
        };

        updateHighlights = function (element, keywords) {
            if (!Array.isArray(keywords)) return;
            
            const keywordMap = new Map(
                keywords.filter(k => k?.words).map(k => [k.words, k])
            );

            $(element).find("." + highlightClass).each(function() {
                const text = this.innerText;
                const newHighlight = keywordMap.get(text);
                
                if (!newHighlight) {
                    removeHighlight(this);
                } else {
                    const currentColor = parseInt(this.className.match(/style-(\d+)/)?.[1] || 0);
                    if (newHighlight.colour !== currentColor) {
                        this.className = this.className.split(' ')
                            .map(cls => cls.startsWith(stylePrefix) ? 
                                 stylePrefix + newHighlight.colour : cls)
                            .join(' ');
                    }
                }
            });
        };

        // 添加新的高亮函数，专门处理新增关键词
        highlightNewKeywords = function(element, newKeywords) {
            if (!Array.isArray(newKeywords) || !newKeywords.length) return;

            try {
                const processedKeywords = newKeywords
                    .filter(item => item?.words && typeof item.words === 'string' && item.words.trim())
                    .sort((a, b) => b.words.length - a.words.length);

                if (!processedKeywords.length) return;

                // 使用 NodeIterator 只遍历未高亮的文本节点
                const iterator = document.createNodeIterator(
                    element,
                    NodeFilter.SHOW_TEXT,
                    {
                        acceptNode: function(node) {
                            if (!node || !node.parentElement) return NodeFilter.FILTER_REJECT;
                            
                            // 过滤掉已高亮节点的子节点
                            let parent = node.parentElement;
                            while (parent && parent !== element) {
                                if (parent.classList?.contains(highlightClass)) {
                                    return NodeFilter.FILTER_REJECT;
                                }
                                parent = parent.parentElement;
                            }
                            
                            // 过滤掉不需要处理的节点
                            if (node.parentElement.tagName && 
                                ['SCRIPT', 'STYLE', 'TEXTAREA'].includes(node.parentElement.tagName)) {
                                return NodeFilter.FILTER_REJECT;
                            }
                            
                            // 过滤掉空文本节点
                            const text = node.nodeValue;
                            return text && text.trim().length > 0 ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_REJECT;
                        }
                    }
                );

                const textNodes = [];
                let currentNode;
                while (currentNode = iterator.nextNode()) {
                    textNodes.push(currentNode);
                }

                // 修改批处理大小设置
                const totalNodes = textNodes.length;
                const batchSize = totalNodes > BATCH_THRESHOLD ? 50 : totalNodes;
                let index = 0;

                function processBatch() {
                    const fragment = document.createDocumentFragment();
                    const end = Math.min(index + batchSize, textNodes.length);
                    
                    for (let i = index; i < end; i++) {
                        const node = textNodes[i];
                        if (!node || !node.parentNode) continue;

                        const text = node.nodeValue;
                        if (!text || !text.trim()) continue;

                        let matches = [];
                        let usedRanges = new Set();
                        
                        processedKeywords.forEach(keyword => {
                            if (!keyword?.words) return;
                            
                            const escapedWord = keyword.words.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
                            const isEnglishWord = /^[A-z_-]+$/.test(keyword.words);
                            const pattern = isEnglishWord ? 
                                new RegExp(`(?:^|(?<=[^A-Za-z]))${escapedWord}(?=[^A-Za-z]|$)`, 'gi') :
                                new RegExp(escapedWord, 'g');
                            
                            let match;
                            while ((match = pattern.exec(text)) !== null) {
                                const matchText = match[0];
                                const start = match.index;
                                const end = start + matchText.length;
                                
                                let shouldHighlight = true;
                                for (let range of usedRanges) {
                                    const [usedStart, usedEnd] = range.split(',').map(Number);
                                    if (!(end <= usedStart || start >= usedEnd)) {
                                        shouldHighlight = false;
                                        break;
                                    }
                                }

                                if (shouldHighlight) {
                                    matches.push({
                                        start: start,
                                        end: end,
                                        text: matchText,
                                        keyword: keyword
                                    });
                                    usedRanges.add(`${start},${end}`);
                                }
                            }
                        });

                        if (matches.length) {
                            matches.sort((a, b) => a.start - b.start);
                            let lastIndex = 0;
                            
                            matches.forEach(match => {
                                if (match.start > lastIndex) {
                                    fragment.appendChild(
                                        document.createTextNode(text.slice(lastIndex, match.start))
                                    );
                                }
                                
                                const highlight = document.createElement('mark');
                                highlight.className = `${highlightClass} ${stylePrefix}${match.keyword.colour || 1}`;
                                highlight.textContent = match.text;
                                highlight.setAttribute('data-highlight-text', match.text);
                                highlight.setAttribute('data-category-index', match.keyword.categoryIndex);
                                
                                fragment.appendChild(highlight);
                                lastIndex = match.end;
                            });

                            if (lastIndex < text.length) {
                                fragment.appendChild(
                                    document.createTextNode(text.slice(lastIndex))
                                );
                            }

                            if (node.parentNode && document.contains(node)) {
                                node.parentNode.replaceChild(fragment, node);
                            }
                        }
                    }
                    
                    index = end;
                    if (index < textNodes.length) {
                        requestAnimationFrame(processBatch);
                    }
                }

                processBatch();

            } catch (error) {
                console.error('Error in highlightNewKeywords:', error);
            }
        };

        // 添加增量更新方法
        function incrementalHighlight(element, keywords, changedNodes) {
            if (!element || !keywords || !keywords.length || !changedNodes.size) return;
            
            try {
                changedNodes.forEach(node => {
                    // 只处理文本节点和未高亮的节点
                    if (node.nodeType === Node.TEXT_NODE || 
                        !node.querySelector('.' + highlightClass)) {
                        highlightNode.call(that, node, keywords, true);
                    }
                });
            } catch (error) {
                console.error('Error in incrementalHighlight:', error);
            }
        }

        function getTextNodes(element) {
            // 检查缓存
            if (nodeCache.has(element)) {
                return nodeCache.get(element);
            }
            
            // 使用优化的遍历
            const nodes = [];
            const iterator = document.createNodeIterator(
                element,
                NodeFilter.SHOW_TEXT,
                { acceptNode: utils.shouldProcessNode }
            );
            
            while (currentNode = iterator.nextNode()) {
                nodes.push(currentNode);
            }
            
            // 缓存结果
            nodeCache.set(element, nodes);
            return nodes;
        }

        // 清除高亮时还原原始节点
        function clearHighlighted(element) {
            const highlights = element.querySelectorAll('.' + highlightClass);
            
            highlights.forEach(highlight => {
                if (document.contains(highlight)) {
                    const originalType = nodeTypes.get(highlight);
                    if (originalType) {
                        // 根据原始类型创建对应节点
                        let newNode;
                        if (originalType.type === Node.TEXT_NODE) {
                            newNode = document.createTextNode(originalType.content);
                        } else {
                            newNode = document.createElement(originalType.tagName);
                            newNode.textContent = originalType.content;
                        }
                        highlight.parentNode.replaceChild(newNode, highlight);
                    } else {
                        utils.mergeTextNodes(highlight);
                    }
                }
                highlight.remove();
                nodeTypes.delete(highlight);
            });
        }

        // 修改返回对象，添加新方法
        return {
            highlight: function (element, keywords, force) {
                if (!element || !keywords || !keywords.length) return;
                try {
                    highlightNode.call(that, element, keywords, force);
                } catch (error) {
                    console.error('Error in highlight:', error);
                }
            },

            highlightNewKeywords: function(element, newKeywords) {
                if (!element || !newKeywords || !newKeywords.length) return;
                try {
                    highlightNewKeywords.call(that, element, newKeywords);
                } catch (error) {
                    console.error('Error in highlightNewKeywords:', error);
                }
            },

            clearHighlighted: function (element) {
                if (!element) return;
                try {
                    const highlights = element.querySelectorAll('.' + highlightClass);
                    highlights.forEach(highlight => {
                        if (document.contains(highlight)) {
                            utils.mergeTextNodes(highlight);
                        }
                        highlight.remove();
                        highlight = null;
                    });
                    if (window.gc) window.gc();
                } catch (error) {
                    console.error('Error in clearHighlighted:', error);
                }
            },

            incrementalHighlight: function(element, keywords, changedNodes) {
                incrementalHighlight(element, keywords, changedNodes);
            },

            removeHighlight: function(element) {
                if (!element) return;
                try {
                    utils.mergeTextNodes(element);
                } catch (error) {
                    console.error('Error in removeHighlight:', error);
                }
            }
        };
    });